/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.activemq.security;

import junit.framework.TestCase;
import org.apache.activemq.jaas.*;
import org.apache.activemq.transport.tcp.StubX509Certificate;
import org.junit.rules.TemporaryFolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.management.remote.JMXPrincipal;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginException;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.*;

public class TextFileCertificateLoginModuleTest extends TestCase {

    private static final String USER_NAME = "10001CN";
    private static final String CERT_USERS_FILE_SMALL = "src/test/resources/cert-users-SMALL.properties";
    private static final String CERT_USERS_FILE_LARGE = "src/test/resources/cert-users-LARGE.properties";
    private static final String CERT_GROUPS_FILE = "src/test/resources/cert-groups.properties";

    private static final Logger LOG = LoggerFactory.getLogger(TextFileCertificateLoginModuleTest.class);
    private static final int NUMBER_SUBJECTS = 10;

    private CertificateLoginModule loginModule;

    public TextFileCertificateLoginModuleTest() {

    }

    protected void setUp() throws Exception {
        loginModule = new TextFileCertificateLoginModule();
    }

    protected void tearDown() throws Exception {
    }

    private void loginTest(String usersFiles, String groupsFile) throws LoginException {


        HashMap options = new HashMap<String, String>();
        options.put("org.apache.activemq.jaas.textfiledn.user", usersFiles);
        options.put("org.apache.activemq.jaas.textfiledn.group",groupsFile);



        JaasCertificateCallbackHandler [] callbackHandlers = new JaasCertificateCallbackHandler[NUMBER_SUBJECTS];
        Subject [] subjects = new Subject[NUMBER_SUBJECTS];

        for(int i=0; i < callbackHandlers.length; i++ ){
           callbackHandlers[i]= getJaasCertificateCallbackHandler("DN=TEST_USER_"+(i+1));
        }

        long startTime = System.currentTimeMillis();

        for(int i=0; i < NUMBER_SUBJECTS; i++){
           Subject subject = doAuthenticate(options, callbackHandlers[i]);
           subjects[i]=subject;

        }

        long endTime =  System.currentTimeMillis();
        long timeTaken = endTime - startTime;



        for(int i= 0; i< NUMBER_SUBJECTS; i++){
            LOG.info("subject is: " + subjects[i].getPrincipals().toString());
        }

        LOG.info( usersFiles + ": Time taken is "+timeTaken);

    }

    private JaasCertificateCallbackHandler getJaasCertificateCallbackHandler(String user) {
        JMXPrincipal principal = new JMXPrincipal(user);
        X509Certificate cert = new StubX509Certificate(principal);
        return new JaasCertificateCallbackHandler(new X509Certificate[]{cert});
    }

    private Subject doAuthenticate(HashMap options, JaasCertificateCallbackHandler callbackHandler) throws LoginException {
        Subject mySubject = new Subject();
        loginModule.initialize(mySubject, callbackHandler, null, options);
        loginModule.login();
        loginModule.commit();
        return mySubject;

    }



    public void testLoginWithSMALLUsersFile() throws IOException {
        try {
            loginTest(CERT_USERS_FILE_SMALL,CERT_GROUPS_FILE);
        } catch (Exception e) {
            LOG.error("Unable to login ", e);
        }

    }

    public void testLoginWithLARGEUsersFile() throws IOException {
        try {
            loginTest(CERT_USERS_FILE_LARGE,CERT_GROUPS_FILE);
        } catch (Exception e) {
            LOG.error("Unable to login ", e);
        }

    }


}
